@@ -1,4 +1,4 @@
-#' @importFrom graphics image.default image par plot title box text axis plot.new plot.window rasterImage layout lcm contour hist
+#' @importFrom graphics image.default image par plot title box text axis plot.new plot.window rasterImage layout lcm contour hist strwidth
 #' @importFrom grDevices dev.capabilities dev.size grey rgb col2rgb
 #' @importFrom utils head tail setTxtProgressBar txtProgressBar packageVersion methods modifyList
 #' @importFrom stats na.omit runif aggregate setNames predict quantile var complete.cases na.pass time
@@ -12,11 +12,6 @@
 #' @import units
 NULL
 
-# re-export:
-#' @importFrom lwgeom st_transform_proj
-#' @export
-lwgeom::st_transform_proj
-
 setOldClass("stars")
 setOldClass("stars_proxy")
 
