@@ -34,7 +34,6 @@ st_as_stars.Raster = function(.x, ..., att = 1, ignore_file = FALSE) {
 
 				return(r)
 			}
-
 		}
 	}
 
@@ -107,10 +106,29 @@ setAs("stars_proxy", "Raster", function(from) {
 	raster::brick(unlist(from))
 })
 
+get_terra_levels = function(x, colors) {
+	# create factor levels, as used by stars, from SpatRaster levels in a data.frame
+	# see https://github.com/r-spatial/stars/pull/484
+	x = x[order(x[[1]]), ] # sort table on level
+	missing_labels = x[[2]] == ""
+	levels = x[[1]]
+	if (any(levels < 0))
+		stop("negative IDs in SpatRaster levels not supported")
+	ex = setdiff(0:max(levels), levels)
+	exclude = rep(FALSE, max(levels) + 1)
+	exclude[ex + 1] = TRUE # 0-based vector
+	list(levels = levels[!missing_labels],
+		 labels = x[[2]][!missing_labels],
+		 exclude = exclude,
+		 colors = colors[!missing_labels])
+}
+
 #' @name st_as_stars
 #' @param ignore_file logical; if \code{TRUE}, ignore the SpatRaster object file name
+#' @param as_attributes logical; if \code{TRUE} and \code{.x} has more than one layer, load these as separate attributes rather than as a band or time dimension (only implemented for the case where \code{ignore_file} is \code{TRUE})
 #' @export
-st_as_stars.SpatRaster = function(.x, ..., ignore_file = FALSE) {
+st_as_stars.SpatRaster = function(.x, ..., ignore_file = FALSE,
+			  as_attributes = all(terra::is.factor(.x))) {
 	if (!requireNamespace("terra", quietly = TRUE))
 		stop("package terra required, please install it first") # nocov
 
@@ -120,13 +138,13 @@ st_as_stars.SpatRaster = function(.x, ..., ignore_file = FALSE) {
 	src = terra::sources(.x, bands=TRUE)
 
 	attr_name = basename(src$source[1])
-	if (attr_name == "")
-		attr_name = "values"
 
 	if (!ignore_file && all(src$source != "")) {
 	# there can be multiple files, but only the first one is used here.
 	# perhaps a warning should be given; better would be to iterate over "sid"
 	# but you might have a situation where some sources are filenames and others are not
+		if (attr_name == "")
+			attr_name = "values"
 		lst = vector("list", length(unique(src$sid)))
 		for (i in unique(src$sid)) {
 			file = unique(src$source[src$sid == i])
@@ -138,7 +156,7 @@ st_as_stars.SpatRaster = function(.x, ..., ignore_file = FALSE) {
 			RasterIO = list(bands = src$bands[src$sid == i])
 			r = try(read_stars(file, RasterIO = RasterIO, ...), silent = TRUE)
 			if (! inherits(r, "try-error")) {
-				if (is.na(st_crs(r)))
+				if (is.na(st_crs(r)) && terra::crs(.x) != "")
 					r = st_set_crs(r, st_crs(terra::crs(.x)))
 				r = fix_dims(r, e)
 				if (length(unique(src$sid)) > 1 && length(dim(r)) > 2)
@@ -157,37 +175,50 @@ st_as_stars.SpatRaster = function(.x, ..., ignore_file = FALSE) {
 		if (!all(is.na(terra::time(.x))))
 			ret = st_set_dimensions(ret, 3, values = terra::time(.x), names = "time")
 
-		return(setNames(ret, attr_name))
+	} else { # ignore_file TRUE:
+		if (terra::nlyr(.x) > 1 && as_attributes) {
+			ret = do.call(c, lapply(seq_len(terra::nlyr(.x)), function(i) st_as_stars(.x[[i]], ignore_file = TRUE)))
+			if (!is.null(names(.x)))
+				names(ret) = names(.x)
+			return(ret) # RETURNS
+		}
+		if (attr_name == "") {
+			if (all(names(.x) == ""))
+				attr_name = "values"
+			else
+				attr_name = paste(names(.x)[1], collapse = ".")
+		}
+		v = terra::values(.x, mat = FALSE)
+		dimv = dim(v) = setNames(dim(.x)[c(2,1,3)], c("x", "y", "band"))
+		if (all(terra::is.factor(.x))) {
+			if (length(terra::levels(.x)) > 1)
+				warning("ignoring categories/levels for all but first layer")
+			colors = try(rgb(terra::coltab(.x)[[1]][-1], maxColorValue = 255), silent = TRUE)
+			if (inherits(colors, "try-error") || length(colors) == 0)
+				colors = NULL
+			if (inherits(l <- terra::levels(.x)[[1]], "data.frame"))
+				l = get_terra_levels(l, colors)
+			else
+				stop("terra levels should return a list of data.frame's; pls update terra")
+			v = structure(factor(as.vector(v), levels = l$levels, labels = l$labels),
+					dim = dimv, colors = l$colors, exclude = l$exclude)
+		}
+		crs = if (terra::crs(.x) == "")
+				NA_crs_
+			else
+				st_crs(terra::crs(.x))
+		dimensions = list(
+				x = create_dimension(from = 1, to = dim(v)[1], offset = e[1],
+							 	delta = (e[2]-e[1])/dim(v)[1], refsys = crs),
+				y = create_dimension(from = 1, to = dim(v)[2], offset = e[4],
+							 	delta = (e[3]-e[4])/dim(v)[2], refsys = crs))
+		dimensions$band = create_dimension(values = names(.x))
+		ret = st_as_stars(list(v), dimensions = create_dimensions(dimensions, get_raster()))
+		if (dim(ret)[3] == 1)
+			ret = adrop(ret, 3)
+		else if (!all(is.na(terra::time(.x))))
+			ret = st_set_dimensions(ret, 3, values = terra::time(.x), names = "time")
 	}
-
-	v = terra::values(.x, mat = FALSE)
-	dim(v) = dim(.x)[c(2,1,3)]
-	if (all(terra::is.factor(.x))) {
-		l = terra::levels(.x)[[1]]
-		colors = try(rgb(terra::coltab(.x)[[1]], maxColorValue = 255))
-		if (inherits(colors, "try-error") || length(colors) == 0)
-			colors = NULL
-		else
-			colors = colors[-length(colors)]
-		v = structure(v + 1, class = "factor", levels = l, colors = colors)
-		# FIXME: should we handle levels for all layers here, or break on multiple different ones?
-	}
-	dimensions = list(
-		x = create_dimension(from = 1, to = dim(v)[1], offset = e[1],
-							 delta = (e[2]-e[1])/dim(v)[1], refsys = st_crs(terra::crs(.x))),
-		y = create_dimension(from = 1, to = dim(v)[2], offset = e[4],
-							 delta = (e[3]-e[4])/dim(v)[2], refsys = st_crs(terra::crs(.x))))
-	dimensions$band = create_dimension(values = names(.x))
-#	l = if (length(names) > 1)
-#		setNames(list(v), deparse(substitute(.x), 50))
-#	else
-#		setNames(list(v), names(.x)[1])
-	ret = st_as_stars(list(v), dimensions = create_dimensions(dimensions, get_raster()))
-	if (dim(ret)[3] == 1)
-		ret = adrop(ret, 3)
-	else if (!all(is.na(terra::time(.x))))
-		ret = st_set_dimensions(ret, 3, values = terra::time(.x), names = "time")
-
 	setNames(ret, attr_name)
 }
 
@@ -256,6 +287,8 @@ fix_dims = function(r, e){
 
 st_as_raster = function(x, class, ...) {
 	stopifnot(inherits(x, "stars"))
+	if (is.null(names(x)))
+		names(x) = make.names(seq_along(x)) # FIXME: why? sdsr_exercises
 	x_crs = st_crs(x)
 	x = st_upfront(x) # x/y dimensions first
 	if (length(dim(x)) > 3) {
@@ -277,22 +310,49 @@ st_as_raster = function(x, class, ...) {
 	dxy = attr(d, "raster")$dimensions
 	stopifnot(all(dxy %in% names(d)))
 	bb = st_bbox(x)
-	values = if (is.factor(x[[1]])) {
-		structure(x[[1]], dim = NULL)
-	} else {
-		as.vector(x[[1]]) # would convert factor into character
-	}
-	if (class == "SpatRaster"){
+	levels = NULL
+	coltab = vector("list", length(x))
+	values = if (all(sapply(x, is.factor))) {
+			ex = attr(x[[1]], "exclude")
+			if (is.null(ex) || class != "SpatRaster")
+				structure(merge(x)[[1]], dim = NULL) # return the factor
+			else {
+				v = vector("list", length(x))
+				levels = vector("list", length(x))
+				for (i in seq_along(v)) {
+					ex = attr(x[[i]], "exclude")
+					ix = which(!ex) - 1 # 0-based index
+					n = as.numeric(structure(x[[i]], dim = NULL)) # factor -> numeric
+					v[[i]] = ix[n]
+					levels[[i]] = data.frame(IDs = ix, categories = levels(x[[i]]))
+					if (!is.null(ct <- attr(x[[i]], "colors"))) {
+						coltab[[i]] = t(col2rgb(rep("#000000", length(ex)), alpha = TRUE))
+						coltab[[i]][which(!ex),] = t(col2rgb(ct, alpha = TRUE))
+					}
+				}
+				do.call(c, v)
+			}
+		} else  {
+			if (any(sapply(x, is.factor)))
+				warning("mix of factor and non-factor attributes: all factor levels are ignored")
+			as.vector(merge(x)[[1]])
+		}
+	if (class == "SpatRaster") {
 		third = setdiff(names(d), dxy)
 		b = terra::rast(nrows = dim(x)[ dxy[2] ], ncols=dim(x)[ dxy[1] ],
 						xmin = bb[1], xmax = bb[3], ymin = bb[2], ymax = bb[4],
 						nlyrs = ifelse(length(dim(x)) == 2, 1, dim(x)[third]),
 						crs = x_crs$wkt)
 		terra::values(b) = values
-		if (all(vapply(x, is.factor, FUN.VALUE = logical(1)))) {
-			terra::coltab(b) = lapply(x, function(x) t(col2rgb(attr(x, "colors"), alpha = TRUE)))
+		if (!is.null(levels)) {
+			levels(b) = levels
+			if (!all(sapply(coltab, is.null)))
+				for (i in seq_len(terra::nlyr(b))) {
+					m = coltab[[i]]
+					terra::coltab(b, layer = i) = cbind(value = seq_len(nrow(m)) - 1, m)
+				}
 		}
-		if (length(dim(x)) != 2){
+		if (length(dim(x)) != 2) {
 			z = seq(d[[third]])
 			if (!any(is.na(z))) {
 				if (is.character(z)) {
@@ -301,7 +361,8 @@ st_as_raster = function(x, class, ...) {
 					names(b) = paste0(third, z)
 				}
 			}
-		}
+		} else if (length(x) == terra::nlyr(b))
+			names(b) = names(x)
 	} else {
 		if (length(dim(x)) == 2) {
 			b = raster::raster(nrows=dim(x)[ dxy[2] ], ncols=dim(x)[ dxy[1] ],
